;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; Program for generating movies
; as described in Ouyed, Pudritz and Clarke 2001
;
; Author: Rachid Ouyed (2001)
;
; To be accessed from JETGET main widget.
;

pro mosaic_event, Event

  common setupvariables

  widget_control, event.id, get_uvalue=uvalue
  widget_control, event.top, get_uvalue=obj

  mystic = uvalue
;
; depending on the dimensions of the dataset, clear or unclear some widgets
;
    if (dim eq 2) or (dim eq 2.5) then begin 
    cclear1=widget_info(event.top, find_by_uname="islice")
    widget_control, cclear1, sensitive=0
    cclear2=widget_info(event.top, find_by_uname="jslice")
    widget_control, cclear2, sensitive=0
    cclear3=widget_info(event.top, find_by_uname="kslice")
    widget_control, cclear3, sensitive=0
    endif
    if dim eq 3 then begin 
    cclear1=widget_info(event.top, find_by_uname="islice")
    widget_control, cclear1, sensitive=1
    cclear2=widget_info(event.top, find_by_uname="jslice")
    widget_control, cclear2, sensitive=1
    cclear3=widget_info(event.top, find_by_uname="kslice")
    widget_control, cclear3, sensitive=1
    endif

;
; what button was pressed
;
  case mystic of
     'xpalette'   : xpalette
     'UpperPanel' : obj->set, variable1 = event.(3)
     'MiddlePanel': obj->set, variable2 = event.(3)
     'LowerPanel' : obj->set, variable3 = event.(3)
     'run'        : obj->Run, mystic
     'PS'         : obj->outfile,  units=0;, /novector
     'MPEG'       : obj->MPEG
     'variables'  : a=obj_new('animvarselect')
     'colortable' : obj->set,colortable=event.(3)
     else         : obj->tool_event, event
  endcase

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;    SLICING SESSION
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO StartMovies, nid, I_slice, J_slice, K_slice, variable1, variable2, variable3
COMMON setupvariables
COMMON images, i_panel1_data, i_panel2_data, i_panel3_data, j_panel1_data, j_panel2_data, j_panel3_data
;
; Get the data
;
      data1=getdata(variable1, nid)
      data2=getdata(variable2, nid)
      data3=getdata(variable3, nid)
;
; Get the data in the selected two dimensional slice
;
if dim eq 3 then begin
      if k_slice eq 0 then j_panel1_data = REFORM(data1[*,J_slice,*])
      if k_slice ne 0 then j_panel1_data = REFORM(data1[*,*,K_slice])
      i_panel1_data = REFORM(data1[I_slice,*,*])      

      if k_slice eq 0 then j_panel2_data = REFORM(data2[*,J_slice,*])
      if k_slice ne 0 then j_panel2_data = REFORM(data2[*,*,K_slice])
      i_panel2_data = REFORM(data2[I_slice,*,*])

      if k_slice eq 0 then j_panel3_data = REFORM(data3[*,J_slice,*])
      if k_slice ne 0 then j_panel3_data = REFORM(data3[*,*,K_slice])
      i_panel3_data = REFORM(data3[I_slice,*,*])
endif
if (dim eq 2) or (dim eq 2.5) then begin
      j_panel1_data = data1
      j_panel2_data = data2
      j_panel3_data = data3
      
endif
END
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;    VISUALIZATION SESSION
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO closeEvent, event
  Widget_Control, event.top, /DESTROY
END

pro animation::run, mystic
;
; When the run or mpeg button is pressed, this procedure is run.
;
COMMON setupvariables
COMMON images, i_panel1_data, i_panel2_data, i_panel3_data, j_panel1_data, j_panel2_data, j_panel3_data
common filmout, movie 
common windowsize, xsize, ysize
common maxmin_1
common time

  default, mystic, " "

;
; Read the widgets
;
      ugh0 = widget_info(self.inbase,find_by_uname='start')
      widget_control,ugh0,get_value=bid
      bid = uint(bid[0])
      ugh1 = widget_info(self.inbase,find_by_uname='last')
      widget_control,ugh1,get_value=eid
      eid = uint(eid[0])
      ugh2 = widget_info(self.inbase,find_by_uname='step')
      widget_control,ugh2,get_value=step
      step = uint(step[0])
      ugh4 = widget_info(self.inbase,find_by_uname='islice')
      widget_control,ugh4,get_value=I_slice
      I_slice = uint(I_slice[0])
      ugh5 = widget_info(self.inbase,find_by_uname='jslice')
      widget_control,ugh5,get_value=J_slice
      J_slice = uint(J_slice[0])
      ugh6 = widget_info(self.inbase,find_by_uname='kslice')
      widget_control,ugh6,get_value=K_slice
      K_slice = uint(K_slice[0])
;
; find the maximum and minimum values for each plot
;
if dim eq 3 then begin
       maxminvar_1, self.variable1, bid, eid, I_slice, 0, tbi=step, along=0
        i1min=fmin
        i1max=fmax
       maxminvar_1, self.variable2, bid, eid, I_slice, 0, tbi=step, along=0
        i2min=fmin
        i2max=fmax
       maxminvar_1, self.variable3, bid, eid, I_slice, 0, tbi=step, along=0
        i3min=fmin
        i3max=fmax
       if k_slice eq 0 then begin
        maxminvar_1, self.variable1, bid, eid, J_slice, 0, tbi=step, along=1
        j1min=fmin
        j1max=fmax
        maxminvar_1, self.variable2, bid, eid, J_slice, 0, tbi=step, along=1
        j2min=fmin
        j2max=fmax
        maxminvar_1, self.variable3, bid, eid, J_slice, 0, tbi=step, along=1
        j3min=fmin
        j3max=fmax
       endif
       if k_slice ne 0 then begin
        maxminvar_1, self.variable1, bid, eid, K_slice, 0, tbi=step, along=1
        j1min=fmin
        j1max=fmax
        maxminvar_1, self.variable2, bid, eid, K_slice, 0, tbi=step, along=1
        j2min=fmin
        j2max=fmax
        maxminvar_1, self.variable3, bid, eid, K_slice, 0, tbi=step, along=1
        j3min=fmin
        j3max=fmax
       endif
;
; find the maximum and minimum values for the axes
;
   Xmini = DimscaleY(0)   
   Xmaxi = DimscaleY(Dimsizes(1)-1)
   ymini = DimscaleZ(0)
   ymaxi = DimscaleZ(Dimsizes(2)-1)
   dimsi1 = dimscaley
   dimsi2 = dimscalez

;
; either j or k is used
;
 if k_slice eq 0 then begin
   Xmin = DimscaleX(0)   
   Xmax = DimscaleX(Dimsizes(0)-1)
   ymin = DimscaleY(0)
   ymax = DimscaleY(Dimsizes(1)-1)
   dims1 = dimscalex
   dims2 = dimscaley
 endif else begin
   Xmin = DimscaleX(0)   
   Xmax = DimscaleX(Dimsizes(0)-1)
   ymin = DimscaleZ(0)
   ymax = DimscaleZ(Dimsizes(2)-1)
   dims1 = dimscalex
   dims2 = dimscalez
 endelse

endif
;
; Same as above, but in the case of two dimensions
;
if (dim eq 2) or (dim eq 2.5) then begin
   maxminvar_1, self.variable1, bid, eid, 0, 0, tbi=step, along=1
   j1min=fmin
   j1max=fmax
   maxminvar_1, self.variable2, bid, eid, 0, 0, tbi=step, along=1
   j2min=fmin
   j2max=fmax
   maxminvar_1, self.variable3, bid, eid, 0, 0, tbi=step, along=1
   j3min=fmin
   j3max=fmax

   Xmin = DimscaleX(0)   
   Xmax = DimscaleX(Dimsizes(0)-1)
   ymin = DimscaleY(0)
   ymax = DimscaleY(Dimsizes(1)-1)
   dims1 = dimscalex
   dims2 = dimscaley

endif
;
; Start the loop
;
FOR nid = bid, eid, step DO BEGIN
   StartMovies, nid, I_slice, J_slice, K_slice, self.variable1,  $ 
           self.variable2, self.variable3
;
; Set the time
;
   time = nid*dt
;
; The plots are in a panel, so make the panel object.
;
   opanel = obj_new('grpanel', columns=fix(dim)-1, rows=3)

mypalette = Obj_New('IDLgrPalette');
mypalette->loadct, self.colortable
;
; Get the right size for the plots. They are scaled relative to the physical
; size of the grid.
;
scysize= ysize
if dim eq 3 then begin
 if k_slice eq 0 then begin
  scxsize=(dimscalex(n_elements(dimscalex)-1)-dimscalex(0))/(dimscaley(n_elements(dimscaley)-1)-dimscaley(0))*ysize/3.
  scx2size=(dimscaley(n_elements(dimscaley)-1)-dimscaley(0))/(dimscalez(n_elements(dimscalez)-1)-dimscalez(0))*ysize/3.
 endif
 if k_slice ne 0 then begin
  scxsize=(dimscalex(n_elements(dimscalex)-1)-dimscalex(0))/(dimscalez(n_elements(dimscalez)-1)-dimscalez(0))*ysize/3.
  scx2size=(dimscalez(n_elements(dimscalez)-1)-dimscalez(0))/(dimscaley(n_elements(dimscaley)-1)-dimscaley(0))*ysize/3.
 endif
endif
if (dim eq 2) or (dim eq 2.5) then begin
 scxsize=(dimscalex(n_elements(dimscalex)-1)-dimscalex(0))/(dimscaley(n_elements(dimscaley)-1)-dimscaley(0))*ysize/3.
 scx2size=(dimscaley(n_elements(dimscaley)-1)-dimscaley(0))/(dimscaley(n_elements(dimscaley)-1)-dimscaley(0))*ysize/3.
endif
;
; The following is for 3-d datasets
;
IF dim eq 3 THEN BEGIN
;
; make the objects for each of the six plots
;
       j_panel1 = obj_new('grcontour', j_panel1_data, max=j1max, min=j1min, $
        xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax, btitle='', ytitle='', xtitle='', $ 
        title='', /color, /fill, /nocolorbar, dims1=dims1, dims2=dims2)
       i_panel1 = obj_new('grcontour', i_panel1_data, max=i1max, min = i1min, $
       xmin=xmini, ymin=ymini, xmax=xmaxi, ymax=ymaxi, title='', ytitle='', xtitle='', $ 
       btitle='', /color, /fill, /nocolorbar, dims1=dimsi1, dims2=dimsi2)
       j_panel2 = obj_new('grcontour', j_panel2_data, max=j2max, min=j2min, $
        xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax, title='', ytitle='', xtitle='', $ 
        btitle='', /color, /fill, /nocolorbar, dims1=dims1, dims2=dims2)
       i_panel2 = obj_new('grcontour', i_panel2_data, max=i2max, min = i2min, $
       xmin=xmini, ymin=ymini, xmax=xmaxi, ymax=ymaxi, title='', ytitle='', xtitle='', $ 
       btitle='', /color, /fill, /nocolorbar, dims1=dimsi1, dims2=dimsi2)
       j_panel3 = obj_new('grcontour', j_panel3_data, max=j3max, min=j3min, $
        xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax, title='', ytitle='', xtitle='', $ 
        btitle='', /color, /fill, /nocolorbar, dims1=dims1, dims2=dims2)
       i_panel3 = obj_new('grcontour', i_panel3_data, max=i3max, min = i3min, $
       xmin=xmini, ymin=ymini, xmax=xmaxi, ymax=ymaxi, title='', ytitle='', xtitle='', $ 
       btitle='', /color, /fill, /nocolorbar, dims1=dimsi1, dims2=dimsi2)
;
; Make the view
;
       viewrect = [xmin, ymin, (xmax-xmin)*(xmax-xmin)/(ymax-ymin), ymax-ymin]
       viewrecti = [xmini, ymini, xmaxi-xmini, ymaxi-ymini]

       j_plotView1 = Obj_New('IDLgrView', Viewplane_Rect=viewRect,  Location=[0,2*SCYSIZE/3])
       i_plotView1 = Obj_New('IDLgrView', Viewplane_Rect=viewRecti,  Location=[SCxSIZE,2*SCYSIZE/3])
       j_plotView2 = Obj_New('IDLgrView', Viewplane_Rect=viewRect,  Location=[0,1*SCYSIZE/3])
       i_plotView2 = Obj_New('IDLgrView', Viewplane_Rect=viewRecti,  Location=[scxsize,1*SCYSIZE/3])
       j_plotView3 = Obj_New('IDLgrView', Viewplane_Rect=viewRect,  Location=[0,0*SCYSIZE/3])
       i_plotView3 = Obj_New('IDLgrView', Viewplane_Rect=viewRecti,  Location=[SCxSIZE,0*SCYSIZE/3])
;
; Title
;
       title1 = obj_new('IDLgrText', self.variable1, font=default_font,location=[xmax*0.98,ymin*0.98], color=white,align=1.0, /onglass)
       title2 = obj_new('IDLgrText', self.variable2, font=default_font,location=[xmax*0.98,ymin*0.98], color=white,align=1.0, /onglass)
       title3 = obj_new('IDLgrText', self.variable3, font=default_font,location=[xmax*0.98,ymin*0.98], color=white,align=1.0, /onglass)
       title4 = obj_new('IDLgrText', 'Time='+strtrim(string(time),1), font=default_font, location=[xmax*0.98, ymax*0.98], color=white, align=1.0, vertical_alignment=1.0, /onglass)
;
; Model
;
       j_imageModel1 = Obj_New('IDLgrModel')
       i_imageModel1 = Obj_New('IDLgrModel')
       j_imageModel2 = Obj_New('IDLgrModel')
       i_imageModel2 = Obj_New('IDLgrModel')
       j_imageModel3 = Obj_New('IDLgrModel')
       i_imageModel3 = Obj_New('IDLgrModel')
; 
; Put everything together
;
       j_imageModel3->Add, j_panel3
       j_imageModel3->Add, title3
       i_imageModel3->Add, i_panel3
       j_imageModel2->Add, j_panel2
       j_imageModel2->Add, title2
       i_imageModel2->Add, i_panel2
       j_imageModel1->Add, j_panel1
       j_imageModel1->Add, title1
       i_imageModel1->Add, i_panel1
       j_imageModel1->Add, title4

       j_plotView3->Add, j_imageModel3
       i_plotView3->Add, i_imageModel3
       j_plotView2->Add, j_imageModel2
       i_plotView2->Add, i_imageModel2
       j_plotView1->Add, j_imageModel1
       i_plotView1->Add, i_imageModel1

       plotView4 = Obj_New('IDLgrView', Viewplane_Rect=viewRect,  Location=[3*SCYSIZE/3,0*SCYSIZE/3])
;
; Scale it to the right size
;
       Self->resize, xstretch=((xmaxi-xmini)/(ymaxi-ymini)*ysize/3+scxsize)/ysize, units=0
       j_imagemodel1->scale, (xmax-xmin)/(ymax-ymin), 1, 1
       j_imagemodel2->scale, (xmax-xmin)/(ymax-ymin), 1, 1
       j_imagemodel3->scale, (xmax-xmin)/(ymax-ymin), 1, 1
       i_imagemodel1->scale, (xmaxi-xmini)/(ymaxi-ymini), 1, 1
       i_imagemodel2->scale, (xmaxi-xmini)/(ymaxi-ymini), 1, 1
       i_imagemodel3->scale, (xmaxi-xmini)/(ymaxi-ymini), 1, 1
;
; put the views into the panel
;
       opanel->add, j_plotView1, /nolocation, units=0, dimensions=[scxsize, ysize/3.]
       opanel->add, i_plotView1, /nolocation, units=0, dimensions=[(xmaxi-xmini)/(ymaxi-ymini)*ysize/3., ysize/3.]
       opanel->add, j_plotView2, /nolocation, units=0, dimensions=[scxsize, ysize/3.]
       opanel->add, i_plotView2, /nolocation, units=0, dimensions=[(xmaxi-xmini)/(ymaxi-ymini)*ysize/3., ysize/3.]
       opanel->add, j_plotView3, /nolocation, units=0, dimensions=[scxsize, ysize/3.]
       opanel->add, i_plotView3, /nolocation, units=0, dimensions=[(xmaxi-xmini)/(ymaxi-ymini)*ysize/3., ysize/3.]

       self->draw, opanel

       myContainer=obj_new('IDL_Container')
       myContainer->Add, j_plotView1
       myContainer->Add, i_plotView1
       myContainer->Add, j_plotView2
       myContainer->Add, i_plotView2
       myContainer->Add, j_plotView3
       myContainer->Add, i_plotView3

ENDIF
;
; for 2-d dataset
;
IF (dim eq 2) or (dim eq 2.5) THEN BEGIN
;
; make the plots
;
       j_panel1 = obj_new('grcontour', j_panel1_data, max=j1max, min=j1min, $
        xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax, title='', ytitle='', xtitle='', $ 
        btitle='', /color, /fill, /nocolorbar, numberoflevels=40)
       j_panel2 = obj_new('grcontour', j_panel2_data, max=j2max, min=j2min, $
        xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax, title='', ytitle='', xtitle='', $ 
        btitle='', /color, /fill, /nocolorbar)
       j_panel3 = obj_new('grcontour', j_panel3_data, max=j3max, min=j3min, $
        xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax, title='', ytitle='', xtitle='', $ 
        btitle='', /color, /fill, /nocolorbar)
;
; the view
;
       viewrect = [xmin, ymin, (xmax-xmin), ymax-ymin]

       j_plotView1 = Obj_New('IDLgrView', Viewplane_Rect=viewRect,  Location=[0,2*SCYSIZE/3])
       j_plotView2 = Obj_New('IDLgrView', Viewplane_Rect=viewRect,  Location=[0,1*SCYSIZE/3])
       j_plotView3 = Obj_New('IDLgrView', Viewplane_Rect=viewRect,  Location=[0,0*SCYSIZE/3])
;
; title
;
       title1 = obj_new('IDLgrText', self.variable1, location=[xmax*0.5,ymin*0.98], align=0.5)
       title2 = obj_new('IDLgrText', self.variable2, location=[xmax*0.5,ymin*0.98], align=0.5)
       title3 = obj_new('IDLgrText', self.variable3, location=[xmax*0.5,ymin*0.98], color=white,align=0.5)
       title4 = obj_new('IDLgrText', 'Time='+strtrim(string(time),1), font=default_font, location=[xmax*0.5,ymax*0.98], color=white, align=0.5, vertical_alignment=1.0, /onglass)
;
; model
;
       j_imageModel1 = Obj_New('IDLgrModel')
       j_imageModel2 = Obj_New('IDLgrModel')
       j_imageModel3 = Obj_New('IDLgrModel')
;
; put everything together
;
       j_imageModel3->Add, j_panel3
       j_imageModel3->Add, title3
       j_imageModel2->Add, j_panel2
       j_imageModel2->Add, title2
       j_imageModel1->Add, j_panel1
       j_imageModel1->Add, title1
       j_imageModel1->Add, title4

       j_plotView3->Add, j_imageModel3
       j_plotView2->Add, j_imageModel2
       j_plotView1->Add, j_imageModel1
;
; Scale it
;
       Self->resize, xstretch=(scxsize)/ysize, units=0
;
; add views to the panel
;
       opanel->add, j_plotView1, /nolocation, units=0, dimensions=[scxsize, ysize/3.]
       opanel->add, j_plotView2, /nolocation, units=0, dimensions=[scxsize, ysize/3.]
       opanel->add, j_plotView3, /nolocation, units=0, dimensions=[scxsize, ysize/3.]

       self->draw, opanel

ENDIF
;
; end of 2-d/3-d if
;
      print, 'File viewed: ID = ', nid

       ENDFOR

      print, 'Animation finished!'

     end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;    WIDGET SESSION
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
FUNCTION animation::init

  COMMON setupvariables

  self.vars = ptr_new(1)

  status = Self->Tool::Init('Mosaic')

  bid        = 000
  eid        = 060
  step       = 1
  I_slice    = Round((Dimsizes(0)-1)/2)
  J_slice    = Round((Dimsizes(1)-1)/2)
  K_slice    = 0
;
; Setup Widgets
;
  firsttext = cw_field(self.inbase, uname='start',value=bid, title='First     ')

  lasttext = cw_field(self.inbase, uname='last', value=eid, title='Last      ')

  steptext = cw_field(self.inbase, uname='step', value=step, title='Increment ')

  islicetext=cw_field(self.inbase, uname='islice', value=I_slice, title='I-Slice   ')

  jslicetext=cw_field(self.inbase, uname='jslice', value=J_slice, title='J-Slice   ')

  kslicetext=cw_field(self.inbase, uname='kslice',value=K_slice, title='K-Slice   ')

  xpalette_button = widget_button(self.inbase, value="Xpalette", uvalue="xpalette", uname="xpalette")
;
; set the self.vars pointer
;
  *self.vars = [varlist]
  if n_elements(extravars) gt 0 then *self.vars = [varlist, extravars]
  if n_elements(variables) gt 0 then *self.vars = [variablelist, varlabels]
;
; the variable droplists
;
  vars1 = FSC_Droplist(self.inbase, Value=*self.vars, Index=0, $
          UValue='UpperPanel',title='Upper  Panel')
  vars2 = FSC_Droplist(self.inbase, Value=*self.vars, Index=6, $
          UValue='MiddlePanel',title='Middle Panel')
  vars3 = FSC_Droplist(self.inbase, Value=*self.vars, Index=3, $
          UValue='LowerPanel',title='Lower  Panel')
;
; buttons
;
  run = widget_button(self.inbase,value='Run',uvalue='run')

  filmtext = widget_button(self.inbase,value='Generate MPEG',uname='makefilm',uvalue='MPEG')

  psbutton = widget_button(self.inbase, value='Save', uvalue='PS')

  helpbutton = widget_button(self.inbase,value='Help', uvalue='Help')

  closeButton = Widget_Button(self.inbase, Value='Close', Event_Pro='closeEvent')

  Self->resize, units=0
;
; some defaults
;
  self.variable1='v1'
  self.variable2='d'
  self.variable3='b1'
  self.colortable = 5
  self.helpfile = "help/HELP_ANIME"
  return, 1

END

pro animation::set, colortable=colortable, variable1=variable1, $
                    variable2=variable2, variable3=variable3

  if n_elements(colortable) ne 0 then self.colortable=colortable+1
  if n_elements(variable1) ne 0 then self.variable1=(*self.vars)[variable1]
  if n_elements(variable2) ne 0 then self.variable2=(*self.vars)[variable2]
  if n_elements(variable3) ne 0 then self.variable3=(*self.vars)[variable3]

end

pro animation__define
    void = {animation, Type: 0, colortable:0, INHERITS tool, $
             animvarselect: obj_new(), variable1: 'v1', variable2: 'd', $
             variable3: 'b1', vars: ptr_new(1)}
end
