;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; Routine to read in the filename to which a file is to be saved
;
; Created Summer 2003
;     by Jan E. Staff
;
; To be called from the JETGET modules
;

pro filenameinput_Event, Event
;
; To buttons, apply or cancel. Which one was pressed?
;
  Widget_Control, event.top, Get_UValue=obj
  widget_control, event.id, get_uvalue=uvalue
  case uvalue of 
    'pstypebutton': begin
                     pstype = event.(4)
                     obj->set, pstype
                   end
    'Apply' : obj->read, event
    'Cancel' : obj->cancel, event
  endcase
end

PRO filenameinput::set, pstype
  if n_elements(pstype) ne 0 then *self.pstype=pstype
END

PRO filenameinput::cancel, event
;
; Close the widget
;
      *self.canceled=1
      widget_control, event.top, /destroy
END

PRO filenameinput::read, event
;
; if the apply button was pressed, this routine is used. It reads in the 
; filename and saves it to *self.filename
;
  widget_control, widget_info(self.modalbase, find_by_uname='file'), get_value=filename
  *self.filename=filename
  widget_control, event.top, /destroy
 
END

FUNCTION filenameinput::canceled
;
; if the cancel button was pressed, this becomes one. the routine that
; called filenameinput can then get to know that the cancel butotn was
; pressed
;
   return, *self.canceled
END

FUNCTION filenameinput::gettype
;
; If ps output is requested, then this differentiate between vector and 
; bitmap images
;
   return, *Self.pstype
END

FUNCTION filenameinput::getfilename
;
; returns the filename the file shall have
;
   return, *Self.filename
END

FUNCTION filenameinput::init, grpleader, type
;
; init routine for filenameinput
;
common setupvariables

self.canceled=ptr_new(0)
self.filename=ptr_new(1)
self.pstype=ptr_new(0)
Self.Modalbase = Widget_Base(column=1, /modal, group_leader=grpleader.base)

filenamelabel = widget_label(self.modalbase, value='Name of '+type +' file to save to: ')
filename = cw_field(self.modalbase, uname='file', title='Filename ', value=outdir)
;
; if ps output file is chosen in setup, then make some radio buttons
; for choosing vector or bitmap ps.
;
if type eq 'postscript' then begin
  pstypebutton = cw_bgroup(self.modalbase, ['Vector', 'Bitmap'], row=1, $
       /exclusive, /no_release, uvalue='pstypebutton', set_value=0 )
endif
;
; the buttons
;
applyButton = Widget_Button(self.modalbase, Value='Apply', uvalue='Apply')
cancelButton = Widget_Button(self.modalbase, Value='Cancel', uvalue='Cancel')
widget_control, self.modalbase, /update, /realize
widget_control, Self.modalbase, set_uvalue=self
xmanager, 'filenameinput', self.modalbase

return ,1

END

pro filenameinput__define
  void = {filenameinput, Type:0, pstype: ptr_new(), canceled: ptr_new(), filename: ptr_new(), INHERITS tool}
end
