;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; 
; IDL Widget Interface Procedures. 

; 
; Written winter 2001
; 

PRO Fluxes_Event, Event
    widget_control, event.id, get_uvalue = uvalue
    widget_control, event.top, get_uvalue=obj
;
; which button
;
    case uvalue of
      'Run'        : obj->Run
      'Fluxes_Type': obj->Set, Type = event.(3)
      else: obj->Tool_Event, event
    endcase
END

PRO Fluxes::Set, Type = Type
; procedure to set variables
  Self.Type = Type
END

PRO Fluxes::Run
;
; If the Run button was pressed, this is being performed
;
  common setupvariables
  common time

;
; Read in from the widgets
;
  BID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FLUXES_BID")
  WIDGET_CONTROL, BID_Ident, GET_VALUE=BID
  BID = LONG(BID[0])

  EID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FLUXES_EID")
  WIDGET_CONTROL, EID_Ident, GET_VALUE=EID
  EID = LONG(EID[0])

  TBI_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FLUXES_TBI")
  WIDGET_CONTROL, TBI_Ident, GET_VALUE=TBI
  TBI = LONG(TBI[0])

  Slice_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="FLUXES_SLICE")
  WIDGET_CONTROL, Slice_Ident, GET_VALUE=Slice
  Slice = LONG(Slice[0])

  Flux = FLTARR((EID-BID)/TBI+1)

  if bid eq eid then begin
    print, "Error: You must specify two different times!"
    stop
  endif

  FOR NID=BID, EID, TBI DO BEGIN
    Data = GetData(Self.Vars[Self.Type]+'Flux', NID)
;
; find the flux for the 3-d case
;
    if dim eq '3' then begin
    CASE Self.Type OF
      0: Flux[(NID-BID)/TBI] = TOTAL(Data[Slice,*,*])
      1: Flux[(NID-BID)/TBI] = FLOAT(DOUBLE(TOTAL(Data[Slice,*,*], /DOUBLE)))
      2: Flux[(NID-BID)/TBI] = TOTAL(Data[Slice,*,*])
      3: Flux[(NID-BID)/TBI] = TOTAL(Data[Slice,*,*])
    ENDCASE
    endif
;
; find the flux for the 2-d case
;
    if (dim eq '2') or (dim eq 2.5) then begin
    CASE Self.Type OF
      0: Flux[(NID-BID)/TBI] = TOTAL(Data[Slice,*])
      1: Flux[(NID-BID)/TBI] = FLOAT(DOUBLE(TOTAL(Data[Slice,*], /DOUBLE)))
      2: Flux[(NID-BID)/TBI] = TOTAL(Data[Slice,*])
      3: Flux[(NID-BID)/TBI] = TOTAL(Data[Slice,*])
    ENDCASE
    endif

   ENDFOR
;
; Make the objects and put them together.
;
  oView = OBJ_NEW('IDLgrView',VIEWPLANE_RECT=[-0.1,0,1.1,1.1])
  oPlot = OBJ_NEW('grPlot', (FINDGEN((EID-BID)/TBI+1)*TBI+BID)*dt, Flux, XTITLE='Time', YTitle=Self.Vars[Self.Type]+'Flux', TITLE=Self.Vars[Self.Type]+'Flux')
  omodel=obj_new('idlgrmodel')
  omodel->add, oplot
  oview->add, omodel

  Self->Draw, oView
END

FUNCTION fluxes::init

  status = Self->Tool::Init('Fluxes')

  Self.Vars = ['Mass', 'Momentum', 'Energy', 'Magnetic']

  TYPE_LIST = FSC_Droplist(Self.InBase, Value=Self.Vars, Index=0, UValue='Fluxes_Type', TITLE = 'Flux :   ')

  FileTitle = Widget_Label(Self.InBase, /ALIGN_LEFT, VALUE='File ID(s)')
;
; Input fields
;
  BID_CW = CW_FIELD(Self.InBase, UNAME = 'FLUXES_BID', TITLE = 'First    ', VALUE = 0)
  EID_CW = CW_FIELD(Self.InBase, UNAME = 'FLUXES_EID', TITLE = 'Last     ', VALUE = 60)
  TBI_CW = CW_FIELD(Self.InBase, UNAME = 'FLUXES_TBI', TITLE = 'Increment', VALUE = 1)
  
  FLUXES_SLICE_CW = CW_FIELD(Self.InBase, UNAME = 'FLUXES_SLICE', TITLE = 'I-Slice  ', VALUE = 30)
;
; Buttons
;
  FLUXES_RUN_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE ='Run', VALUE='Run')

  FLUXES_PRINT_BUTTON = WIDGET_BUTTON(Self.InBase, UValue = 'PS', VALUE='Save')
  
  FLUXES_HELP_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE = 'Help', VALUE='Help')
  
  FLUXES_CLOSE_BUTTON = WIDGET_BUTTON(Self.InBase, UValue='Close', VALUE='Close')

  widget_control, self.base, /update

  Self.helpfile = "help/HELP_FLUXES"
  Self->Resize, xstretch=1.2
  return, 1
end

PRO fluxes__define
  void = {fluxes, Type: 0, Vars: strarr(4), INHERITS tool}
END
