;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; Contour object which expands the functionality of IDL's buildin object.
; Created Summer 2001
;   July,  2001, M. S. Joergensen
;
; Modification history:
;
; To be called from some different JETGET routines
;

FUNCTION grPlot::FormatTick, mark

end

FUNCTION grPlot::INIT, X, Y, XTITLE=XTITLE, YTITLE=YTITLE, TITLE=TITLE, $
	Color=Color, _EXTRA = e

    IF N_ELEMENTS(TITLE)  EQ 0 THEN TITLE  = ''
    IF N_ELEMENTS(XTITLE) EQ 0 THEN XTITLE = ''
    IF N_ELEMENTS(YTITLE) EQ 0 THEN YTITLE = ''

    IF (self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0

    Self.oPlot = OBJ_NEW('IDLgrPlot', X, Y)

    Self.oPlot->GetProperty, XRANGE=Xr, YRANGE=Yr

    i = 0
    WHILE((Xr[1]-Xr[0])*10^i le 1) DO i = i + 1

    j = 0
    WHILE((Yr[1]-Yr[0])/10^j GT 10) DO j = j + 1

    if (i eq 0) then WHILE((Xr[1]-Xr[0])*10^i le 0) DO i = i - 1

    if ((j lt -1) or (j gt 1)) then begin
      YTITLE = YTITLE + ' / 10^' + STRING(j, FORMAT='(i1)')

      Y = Y/10^j
    endif

    Self.oPlot->SetProperty, DataX = X
    Self.oPlot->SetProperty, DataY = Y

    Self.oPlot->GetProperty, XRange = Xr
    Self.oPlot->GetProperty, YRange = Yr

    if (i ne 0) then $
      XTitle = XTITLE + ' / 10^' + STRING(i, FORMAT='(i1)')
    oXTitle = OBJ_NEW('IDLgrText', XTITLE)
    oXTitle->GetProperty, CHAR_DIMENSIONS=CHARDIMENSIONS
    oXTitle->SetProperty, CHAR_DIMENSIONS=CHARDIMENSIONS*.5

    oYTitle = OBJ_NEW('IDLgrText', YTITLE)
    oYTitle->GetProperty, CHAR_DIMENSIONS=CHARDIMENSIONS
    oYTitle->SetProperty, CHAR_DIMENSIONS=CHARDIMENSIONS*.5

    if yr[1] ne yr[0] then begin
      oXAxis = OBJ_NEW('IDLgrAxis', 0, RANGE=Xr, Ticklen = (Yr[1]-Yr[0])*0.05, LOCATION=[0, Yr[0],0],/EXACT,TITLE=oXTitle)
      oYAxis = OBJ_NEW('IDLgrAxis', 1, RANGE=Yr, Ticklen = (Xr[1]-Xr[0])*0.05, LOCATION=[Xr[0],0,0],/EXACT,TITLE=oYTitle) 
    endif else begin
      yr[0]=yr[0]*0.9
      yr[1]=yr[1]*1.1
      oYAxis = OBJ_NEW('IDLgrAxis', 1, RANGE=Yr, Ticklen = (Xr[1]-Xr[0])*0.05, LOCATION=[Xr[0],0,0],/EXACT,TITLE=oYTitle)
     
      oXAxis = OBJ_NEW('IDLgrAxis', 0, RANGE=Xr, Ticklen = (Yr[1]-Yr[0])*0.05, LOCATION=[0, Yr[0],0],/EXACT,TITLE=oXTitle)
    endelse
      

    oTitle = OBJ_NEW('IDLgrText', Title, vertical_alignment=0.0, alignment=0.5, location=[(max(x)-min(x))/2+min(x), max(y)])

    Self->Add, Self.oPlot
    Self->Add, oXAxis
    Self->Add, oYAxis

    Self->Translate, -Xr[0], -Yr[0], 0
    Self->Scale, 1./(Xr[1]-Xr[0]), 1./(Yr[1]-Yr[0]), 1.
    Self->Scale, 0.75, 0.8, 1.
    Self->Translate, .18, .15, 0

    return, 1
END

PRO grPlot__define

  struct = { grPlot, INHERITS IDLgrModel, oPlot: OBJ_NEW() }

END
