;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; 
; IDL Widget Interface Procedures. 

; 
; Written winter 2001 by Michael Joergensen
; 
; To be called from JETGET main widget
;

PRO Isosurfaces_Event, Event

  common setupvariables

  widget_control, event.id, get_uvalue = uvalue
  widget_control, event.top, get_uvalue=obj

;
; What button was pressed
;
  case uvalue of
    'filled'         : begin
                        obj->set, filled=event.(4)
                       end
    'Run'            : BEGIN
      obj->Reset_Panel, 1, 1
      obj->Run, uvalue
    END
    'PANEL'          : BEGIN
      obj->Reset_Panel, N_Columns, N_Rows
      obj->Run, uvalue
    END
    'Isosurface_Type': BEGIN
      obj->Set, Type = event.(3)
      obj->Up, event.(3)
    END
    'Isosurface_Var' : obj->Set, Var  = event.(3)
    'Isosurface_CT'  : obj->Set, CT   = event.(3)
    'Ax1_Slider' : obj->Set, Ax1 = event.(3)
    'Ax2_Slider' : obj->Set, Ax2 = event.(3)
    'Ax3_Slider' : obj->Set, Ax3 = event.(3)
    else             : obj->Tool_Event, event
  endcase
END

PRO Isosurface::Set, filled=filled, Type = Type, CT = CT, Var = Var, Ax1 = Ax1, Ax2 = Ax2, Ax3 = Ax3
;
; procedure to set some variables
;
  if n_elements(filled) NE 0 then self.filled = filled
  if n_elements(Type)   NE 0 then Self.Type = Type
  if n_elements(CT)     NE 0 then Self.CT = CT
  if n_elements(Var)    NE 0 then Self.Var = Var
  if n_elements(Ax1)    NE 0 then Self.Ax1 = Ax1
  if n_elements(Ax2)    NE 0 then Self.Ax2 = Ax2
  if n_elements(Ax3)    NE 0 then Self.Ax3 = Ax3
END

PRO Isosurface::Up, Run_Type
;
; procedure to clear or unclear some widgets depending on whether a single
; file or multiple files should be shown
;
  CASE Run_Type OF
    0        : BEGIN
    		 SURFACE_EID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_EID")
		 WIDGET_CONTROL, SURFACE_EID_Ident, SENSITIVE = 0
    		 SURFACE_TBI_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_TBI")
		 WIDGET_CONTROL, SURFACE_TBI_Ident, SENSITIVE = 0
    		 SURFACE_VRML_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_VRML")
		 WIDGET_CONTROL, SURFACE_VRML_Ident, SENSITIVE = 1
    		 SURFACE_MPEG_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_MPEG")
		 WIDGET_CONTROL, SURFACE_MPEG_Ident, SENSITIVE = 0
    		 SURFACE_PANEL_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_Panel")
		 WIDGET_CONTROL, SURFACE_PANEL_Ident, SENSITIVE = 0
	       END
    1        : BEGIN
    		 SURFACE_EID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_EID")
		 WIDGET_CONTROL, SURFACE_EID_Ident, SENSITIVE = 1
    		 SURFACE_TBI_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_TBI")
		 WIDGET_CONTROL, SURFACE_TBI_Ident, SENSITIVE = 1
    		 SURFACE_VRML_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_VRML")
		 WIDGET_CONTROL, SURFACE_VRML_Ident, SENSITIVE = 0
    		 SURFACE_MPEG_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_MPEG")
		 WIDGET_CONTROL, SURFACE_MPEG_Ident, SENSITIVE = 1
    		 SURFACE_PANEL_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_Panel")
		 WIDGET_CONTROL, SURFACE_PANEL_Ident, SENSITIVE = 1
    	       END
  END

END

PRO Isosurface::Run, uvalue
;
; when run, vrml, panel or mpeg button is pressed, this routine is being run
;
  common setupvariables
  common time

 default, uvalue, " "
 if obj_valid(Self.oMPEG) then self.frame=0
;
; read input from the widgets
;
  BID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_BID")
  WIDGET_CONTROL, BID_Ident, GET_VALUE=BID
  BID = LONG(BID[0])

  EID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_EID")
  WIDGET_CONTROL, EID_Ident, GET_VALUE=EID
  EID = LONG(EID[0])

  TBI_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_TBI")
  WIDGET_CONTROL, TBI_Ident, GET_VALUE=TBI
  TBI = LONG(TBI[0])

  SURFACE_LEVEL_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="ISOSURFACE_LEVEL")
  WIDGET_CONTROL, SURFACE_LEVEL_Ident, GET_VALUE=Level
;
; Level is the level at which to draw the surface. Level is read in as a
; string, but is changed to a float, since that's how it is going to be used.
; However, that ruins the knowledge of the number of digits requested by the
; user (float(3.2)=3.20000), so the string part of level is kept in level2 for
; printout to the screen.
;
  level2=level
  Level = FLOAT(Level[0])

  IF (Self.Type EQ 0) THEN $
    EID = BID

  Self.Varname = (*Self.Vars)[Self.Var]
;
; generate the panel if necessary
;
  if uvalue eq 'PANEL' then begin
    opanel = obj_new('grpanel', columns=n_columns, rows=n_rows)
  endif

  donebefore=0    ;variable to be used if first dataset doesn't contain given level
;
; the loop
;
  FOR NID=BID, EID, TBI DO BEGIN
;
; get the data
;
    Data = GetData(Self.Varname, NID)

    S = Size(Data)
;
; make the surface
;
    SHADE_VOLUME, Data, Level, c, d

    IsoSurf = OBJ_NEW('grIsoSurface', Data, Level, c, d, self.filled)
;
; rotate it
; 
;    IsoSurf->Rotate, [1,0,0], Self.Ax3
;    IsoSurf->Rotate, [0,1,0], -Self.Ax2
;    IsoSurf->Rotate, [0,0,1], Self.Ax1
    IsoSurf->Rotate, [1,0,0], Self.Ax1
    IsoSurf->Rotate, [0,1,0], -Self.Ax2
    IsoSurf->Rotate, [0,0,1], -Self.Ax3
;    IsoSurf->Rotate, [0,0,1], -90
 
    if (NID eq BID) or (donebefore eq 0) then begin
      if n_elements(c) ne 0 then donebefore = 1
      z = 0
      IsoSurf->GetProperty, Transform = T
      cor = fltarr(2,8)
;
; project the object into a plane
;
      if  (z eq 0) then begin
        cor[*,0] = CVT_TO_2D(0,0,0,T)
        cor[*,1] = CVT_TO_2D(0,0,S[3]-1,T)
        cor[*,2] = CVT_TO_2D(0,S[2]-1,0,T)
        cor[*,3] = CVT_TO_2D(0,S[2]-1,S[3]-1,T)
        cor[*,4] = CVT_TO_2D(S[1]-1,0,0,T)
        cor[*,5] = CVT_TO_2D(S[1]-1,0,S[3]-1,T)
        cor[*,6] = CVT_TO_2D(S[1]-1,S[2]-1,0,T)
        cor[*,7] = CVT_TO_2D(S[1]-1,S[2]-1,S[3]-1,T)
      end else begin
        x1min = IsoSurf->GetX1Min()
        x1max = IsoSurf->GetX1Max()
        x2min = IsoSurf->GetX2Min()
        x2max = IsoSurf->GetX2Max()
        x3min = IsoSurf->GetX3Min()
        x3max = IsoSurf->GetX3Max()
        cor[*,0] = CVT_TO_2D(x1min,x2min,x3min,T)
        cor[*,1] = CVT_TO_2D(x1min,x2min,x3max,T)
        cor[*,2] = CVT_TO_2D(x1min,x2max,x3min,T)
        cor[*,3] = CVT_TO_2D(x1min,x2max,x3max,T)
        cor[*,4] = CVT_TO_2D(x1max,x2min,x3min,T)
        cor[*,5] = CVT_TO_2D(x1max,x2min,x3max,T)
        cor[*,6] = CVT_TO_2D(x1max,x2max,x3min,T)
        cor[*,7] = CVT_TO_2D(x1max,x2max,x3max,T)
      end
    end
;
; find the corners and the length of the object
;
    if  (z ne 0) then begin
      c1 = Min(cor(0,*))
      c2 = Min(cor(1,*))/z
      l1 = (Max(cor(0,*))-Min(cor(0,*)))/z
      l2 = (Max(cor(1,*))-Min(cor(1,*)))/z
    end else begin
      c1 = Min(cor(0,*))
      c2 = Min(cor(1,*))
      l1 = (Max(cor(0,*))-Min(cor(0,*)))
      l2 = (Max(cor(1,*))-Min(cor(1,*)))
    end

    oModel = OBJ_NEW('IDLgrModel')
;
; change the font if panel is being used
;
    if uvalue eq 'PANEL' then $
      ofont = obj_new('idlgrfont', 'Helvetica', size=14/n_rows[0]) $
    else $
      ofont = obj_new('idlgrfont', 'Helvetica', size=14)
;
; titles
;
    oTitle = OBJ_NEW('IDLgrText', $
       strings=Self.Varname+'-surface'+' level = '+strtrim(string(level2[0])), $
       locations=[l1*.5+c1,l2*.98+c2],/onglass, VERTICAL_ALIGNMENT=1.0, $
       alignment=0.5, font=ofont)

    oTitle2 = OBJ_NEW('IDLgrText', $
       strings='Time = '+string(dt*NID, format='(i3)'),  $ 
       locations=[c1+0.98*l1,c2+0.02*l2],/onglass, VERTICAL_ALIGNMENT=0.0, $
       alignment=1.0, font=ofont)

    oModel->Add, oTitle
    oModel->Add, oTitle2
    omodel->setproperty, lighting=1
;
; view
;
    oView = OBJ_NEW('IDLgrView', VIEWPLANE_RECT=[c1, c2, l1+1, l2], ZClip = [S[1], -S[1]], Eye = S[1] + 10)
    olight=obj_new('idlgrlight', type=1, location=[c1+c2, l1+l2, s[1]+10])
    omodel->add, olight

    omodel->Add, IsoSurf
    oView->Add, oModel

    if uvalue eq 'PANEL' then begin
      opanel->add, oview
    endif else begin
      Self->resize,xstretch=1.0
      Self->Draw, oView
    endelse
  ENDFOR

  if uvalue eq 'PANEL' then begin
    self->resize, xstretch=n_columns/float(n_rows)
    self->draw, opanel
  endif
  print, "Finished Isosurface."
END

FUNCTION isosurface::init

COMMON setupvariables
COMMON new_variables

  status = Self->Tool::Init('Isosurfaces')
;
; the single/multiple droplist
;
  TYPE_LIST = FSC_Droplist(Self.InBase, Value=['Single','Multiple'], Index=0, UValue='Isosurface_Type', TITLE = 'File(s) :   ')

;
; set the self.vars pointer
;
  Self.vars = ptr_new(1)

  *Self.Vars = [varlist]
  if n_elements(extravars) gt 0 then *Self.Vars = [varlist, extravars]
  if n_elements(variables) gt 0 then *Self.Vars = [*Self.Vars, varlabels]
;
; the variable droplist
;
  SURFACE_Var_LIST = FSC_Droplist(Self.InBase, Value=*Self.Vars, Index=0, UVALUE='Isosurface_Var', TITLE = 'Variable :  ')

  FileTitle = Widget_Label(Self.InBase, /ALIGN_LEFT, VALUE='File ID(s)')
;
; input fields
;
  BID_CW = CW_FIELD(Self.InBase, UNAME = 'ISOSURFACE_BID', TITLE = 'First    ', VALUE = 0)
  EID_CW = CW_FIELD(Self.InBase, UNAME = 'ISOSURFACE_EID', TITLE = 'Last     ', VALUE = 60)
  TBI_CW = CW_FIELD(Self.InBase, UNAME = 'ISOSURFACE_TBI', TITLE = 'Increment', VALUE = 1)
;
; the sliders
;
  SURFACE_LEVEL_CW = CW_FIELD(Self.InBase, UNAME = 'ISOSURFACE_LEVEL', TITLE = 'Level    ', VALUE = '0.1')
  GEOM = WIDGET_INFO(Self.InBase,/GEOMETRY)
  XSize = GEOM.Scr_XSize

  rotationlabel = widget_label(self.Inbase, value='Rotation around axis: ', /align_left)

  Ax1_slider = WIDGET_SLIDER(Self.InBase, UVALUE = 'Ax1_Slider', TITLE = 'x1', Minimum=0, Maximum=90, Value=Self.Ax1, XSize = XSize)

  Ax2_slider = WIDGET_SLIDER(Self.InBase, UVALUE = 'Ax2_Slider', TITLE = 'x2', Minimum=0, Maximum=90, Value=Self.Ax2, XSize = XSize)

  Ax3_slider = WIDGET_SLIDER(Self.InBase, UVALUE = 'Ax3_Slider', TITLE = 'x3', Minimum=0, Maximum=90, Value=Self.Ax3, XSize = XSize)
;
; radio buttons
;
  stylelabel = widget_label(self.inbase, /align_left, value='Style')
  fillbuttons = cw_bgroup(self.inbase, ['Lines', 'Filled'], row=1, /exclusive, /no_release, uvalue='filled', uname='filled')
;
; buttons
;
  ISOSURFACE_RUN_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE ='Run', VALUE='Run')

  SURFACE_PRINT_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='PS', VALUE='Save')

  SURFACE_VRML_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='VRML', VALUE='VRML', UNAME='ISOSURFACE_VRML')

  SURFACE_PANEL_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='PANEL', VALUE='Panel',UNAME='ISOSURFACE_Panel')

  SURFACE_MPEG_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='MPEG', VALUE='MPEG',UNAME='ISOSURFACE_MPEG')
  
  SURFACE_HELP_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE='Help',VALUE='Help')

  ISOSURFACE_CLOSE_BUTTON = WIDGET_BUTTON(Self.InBase, UValue='Close', Value='Close')

  Self->Up, 0

  widget_control, self.base, /update
  Self->Resize
  self.Helpfile="help/HELP_ISOSURFACE"
  return, 1
end

PRO isosurface__define
  void = {isosurface, filled: 0, Type: 0, Var: 0, Vars: ptr_new(1), CT: 0, Ax1: 0, Ax2: 0, Ax3: 0, INHERITS tool}
END
