;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; 
; IDL Widget Interface Procedures. 

; 
; Written winter 2001
; 

PRO Quantities_Event, Event
    widget_control, event.id, get_uvalue = uvalue
    widget_control, event.top, get_uvalue=obj

    case uvalue of
      'Quantities_flines' : obj->set, flines = event.(3)
      'Quantities_Type'   : obj->set, droplist = event.(3)
      'Run'               : obj->Run
      'Quantities_Type'   : obj->Set, Type = event.(3)
      else                : obj->Tool_Event, event
    endcase
END

PRO Quantities::Set, Type = Type, droplist = droplist, flines = flines
  if n_elements(flines) ne 0 then self.flines = flines
  if n_elements(type) ne 0 then Self.Type = Type
  if n_elements(droplist) ne 0 then Self.Droplist = droplist
END

PRO Quantities::Run
common time

  COMMON setupvariables

;
; Read the widgets
;
  BID_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="QUANTITIES_BID")
  WIDGET_CONTROL, BID_Ident, GET_VALUE=BID
  BID = LONG(BID[0])

  Sx_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="QUANTITIES_Sx")
  WIDGET_CONTROL, Sx_Ident, GET_VALUE=Sx
  sx=sx[0]
  Sy_Ident = WIDGET_INFO(Self.Base, FIND_BY_UNAME="QUANTITIES_Sy")
  WIDGET_CONTROL, Sy_Ident, GET_VALUE=Sy
  sy=sy[0]

  WIDGET_CONTROL, WIDGET_INFO(Self.Base, Find_by_uname='QUANTITIES_steplen'), get_value=steplen
  self.steplen=float(steplen[0])

  WIDGET_CONTROL, WIDGET_INFO(Self.Base, Find_by_uname='QUANTITIES_nsteps'), get_value=nsteps
  nsteps=long(nsteps[0])

; 
; Get the data
;
  vec1data = getdata((*Self.Vecs)[self.flines]+'1', bid)
  vec2data = getdata((*Self.Vecs)[self.flines]+'2', bid)
  vec3data = getdata((*Self.Vecs)[self.flines]+'3', bid)

  Sz = 3.
;
; Make the field line
;
  Lines = OBJ_NEW('grFieldlines', vec1Data, vec2Data, vec3Data, steplen=self.steplen, Sx=Sz, Sy=Sy, Sz=Sx, nSteps=nsteps, DimScaleX=DimScaleX, DimScaleY=DimScaleY, DimScaleZ=DimScaleZ)
  Lines->GetProperty, Fieldline=Fieldline, Sy=Sy, Sz=Sx, /indicies

;
; Calculate the quantity along the field line
;
    data = Getdata((*self.vars)[self.droplist], bid)
    Quant = INTERPOLATE(data, REFORM(FieldLine[0,*]), REFORM(FieldLine[1,*]), REFORM(FieldLine[2,*]))
    YTITLE = string((*self.vars)[self.droplist])

  N = UINT(N_ELEMENTS(Fieldline)/3.)
  fl1=lines->index2xunit(fieldline[0,*])
  fl2=lines->index2yunit(fieldline[1,*])
  fl3=lines->index2zunit(fieldline[2,*])
  s=total([0,reform(sqrt((fl1[1:N-1]-fl1[0:N-2])^2+(fl2[1:N-1]-fl2[0:N-2])^2+(fl3[1:N-1]-fl3[0:N-2])^2))],/cumulative)
;  S = TOTAL([0, REFORM(SQRT((Fieldline[0,1:N-1]-Fieldline[0,0:N-2])^2+(Fieldline[1,1:N-1]-Fieldline[1,0:N-2])^2+$
;                                                                (Fieldline[2,1:N-1]-Fieldline[2,0:N-2])^2))],/CUMULATIVE)
  obj_destroy, lines

  XTitle = 'S'
;
; Make the objects, and put them together
;
  OTITLE = obj_new('idlgrtext', 'Time = '+strtrim(bid*dt,1), $
      vertical_alignment=1.0, alignment=1.0, location=[(1.1+0.1)/2, 1.1])
  oPlot = OBJ_NEW('grPlot', S, Quant, XTITLE=XTITLE, YTITLE=YTITLE)

  omodel = obj_new('idlgrmodel')

  omodel->add, oplot
  omodel->add, otitle

  oView = OBJ_NEW('IDLgrView', VIEWPLANE_RECT=[-0.1,0,1.1,1.1])

  oview->add, omodel

  Self->Draw, oView

END

FUNCTION quantities::init
  common new_variables
  common setupvariables

  self.steplen=0.001
  self.nsteps=10000

  status = Self->Tool::Init('Quantities')
;
; Make the variables pointer point the right place
;
  Self.Vecs = ptr_new(1)
  if n_elements(ownveclabels) gt 0 then begin
   if n_elements(veclabels2) ne 0 then begin
     *Self.Vecs = [ownveclabels(where(ownveclabels ne '0')), $
       veclabels2(where(veclabels2 ne '0'))]
   endif else begin
     if (where(ownveclabels ne '0'))[0] ne -1 then begin
       *Self.Vecs = [ownveclabels(where(ownveclabels ne '0'))]
     endif
   endelse
  endif else begin
    *Self.Vecs = ['b','v']
  endelse
    if n_elements(veclabels2) ne 0 then begin
     if (where(veclabels2 ne '0' and veclabels2 ne ''))[0] ne -1 then begin
       if n_elements(*self.vecs) ne 0 then begin
         *Self.Vecs = ['b','v',veclabels2(where(veclabels2 ne '0' and veclabels2 ne ''))]
       endif else begin
         *Self.Vecs = [veclabels2(where(veclabels2 ne '0' and veclabels2 ne ''))]
       endelse
     endif
    endif

  self.vars = ptr_new(1)
  *Self.Vars = [varlist]
  if n_elements(extravars) gt 0 then *Self.Vars = [varlist, extravars]
  if n_elements(variables) gt 0 then begin
    *Self.Vars = [*Self.Vars, varlabels]  
  endif

;
; The widgets
;
  TYPE_LIST1 = FSC_Droplist(Self.InBase, Value=*Self.vecs, Index=0, UValue='Quantities_flines', TITLE = 'Vector   :   ')

  TYPE_LIST2 = FSC_Droplist(Self.InBase, Value=*Self.vars, Index=0, UValue='Quantities_Type', TITLE = 'Variable :   ')

  BID_CW = CW_FIELD(Self.InBase, UNAME = 'QUANTITIES_BID', TITLE = 'File ID    ', VALUE = 0)

  StartTitle = Widget_Label(Self.InBase, /ALIGN_LEFT, VALUE='Footpoints coordinates')

  Sx_CW = CW_FIELD(Self.InBase, UNAME = 'QUANTITIES_Sx', TITLE = 'Sx         ', VALUE = 8)
  Sy_CW = CW_FIELD(Self.InBase, UNAME = 'QUANTITIES_Sy', TITLE = 'Sy         ', VALUE = 8)

  steplen_CW = CW_FIELD(Self.inbase, uname='QUANTITIES_steplen', Title='Step length', value=self.steplen)

  nsteps_CW = CW_FIELD(Self.inbase, uname='QUANTITIES_nsteps', Title='# Steps    ', value=self.nsteps)
  
  QUANTITIES_RUN_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE ='Run', VALUE='Run')

  QUANTITIES_HELP_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE = 'Help', VALUE='Help')
  
  QUANTITIES_PRINT_BUTTON = WIDGET_BUTTON(Self.InBase, UValue = 'PS', VALUE='Save')
  
  QUANTITIES_HELP_BUTTON = WIDGET_BUTTON(Self.InBase, UVALUE = 'Help', VALUE='Help')
  
  QUANTITIES_CLOSE_BUTTON = WIDGET_BUTTON(Self.InBase, UValue='Close', VALUE='Close')

  widget_control, self.base, /update
  Self.Helpfile="help/HELP_QUANTITIES"
  Self->Resize, xstretch=1.2
  return, 1
end

PRO quantities__define
  void = {quantities, nsteps: 1000L, steplen: 0.001, flines: 0, droplist: 0, vecs: ptr_new(1), vars: ptr_new(1), Type: 0, INHERITS tool}
END
