;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; Procedure readascii
;
; Parameters:
;   
;   nid: the file number to read.
;   count: the number this variable is in the file.
;   variable: output parameter, holding the content of the read data.
;   filename: optional parameter holding the name of the file to read
;             if this is used, nid will be neglected.
;
; Generated summer 2003 by Jan Staff
;
; Procedure for reading ascii data files to be used in JETGET.
; The ascii files are supposed to be constructed in the following way:
;
; Note: the number of dimensions should be given in the JETGET Setup module.
;   1) The size of the grid, for instance 60x60x60 zones. This should be an
;      array containing the the two (three) sizes.
;   2) The scaling in each direction. This is a one dimensional array with
;      the size of the grid. So two (three) of these arrays are needed.
;   3) The actual data-arrays, in the order given in JETGETs Structure module.
;
; Note that this file will read all variables in the file until it finds the
; requested variable.
;

pro readascii, nid, count, variable, filename=filename

common setupvariables

BaseName = DataDir[0]+'/'+BasePrefix[0]
if n_elements(filename) eq 0 then $
  asciifilename = Numberfilename(BaseName, NID, Basepostfix[0], nidlength=n_digits[0]) $
else $
  asciifilename = datadir[0]+'/'+Filename[0]

get_lun, lun
openr, lun, asciifilename

dimsizes=intarr(dim[0])
readf, lun, dimsizes

dimscalex = fltarr(dimsizes[0])
dimscaley = fltarr(dimsizes[1])
if dim[0] eq 3 then $
dimscalez = fltarr(dimsizes[2])

readf, lun, dimscalex
readf, lun, dimscaley
readf, lun, dimscalez

var=fltarr(dimsizes[0], dimsizes[1], dimsizes[2])

for i=1,count do begin
  readf, lun, var
endfor
variable=var
close, lun
end
