;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; 
; JETGET-tool for visualising variables in a cut through a 3-D file, or
; visualising the variables in a 2-D file.
;
; Written Summer 2001 by Jan Staff
;
; To be called from JETGET main widget.
;

PRO slices_event, Event

    widget_control, event.id, get_uvalue = uvalue
    widget_control, event.top, get_uvalue=obj

    ud=widget_info(event.top, find_by_uname="wid_button_34")
    widget_control, ud, get_value=cs

;
; What selection has been made:
;
    case uvalue of
      'Run'                     : obj->Run, uvalue
      'PANEL'                   : begin
                         obj->reset_panel, n_columns, n_rows
                         obj->Run, uvalue
      end
      'xpalette'                : xpalette
      'Slices_droplist'         : obj->Set, droplist = event.(3)
      'Slices_palette_droplist' : obj->Set, palette_droplist = event.(3)
      'wid_button_34'           : begin
                                  obj->Set, cs = event.(4)
                                  obj->up
                                  end
;
; The following two lines have been inserted to avoid confusion for tool.
;
      'wid_button_36'           : begin
                                  end
      'wid_button_50'           : begin
                                  end
      'MPEG'                    : begin
                                    obj->MPEG
                                  end
    else                        : obj->Tool_Event, event
  endcase
END

PRO slices::run, uvalue
;
; Procedure that is started when the Run, Panel or MPEG button is pressed
;
 Common maxmin_1
 common time
 common colors, b_curr, b_orig, g_curr, g_orig, r_curr, r_orig
 common setupvariables

 default, uvalue, " "
 if obj_valid(Self.oMPEG) then self.frame=0

;
; Read input values
;

 yd=widget_info(self.inbase, find_by_uname="wid_button_36")
 widget_control, yd, get_value=aa
 along = aa
 first=widget_info(Self.inbase, find_by_uname="wid_text_30") 
 widget_control, first, get_value=bid
 last=widget_info(Self.inbase, find_by_uname="wid_text_31") 
 widget_control, last, get_value=eid
 step=widget_info(Self.inbase, find_by_uname="wid_text_32") 
 widget_control, step, get_value=tbi
 first=widget_info(Self.inbase, find_by_uname="wid_text_46")
 widget_control, first, get_value=beginslice
 last=widget_info(Self.inbase, find_by_uname="wid_text_47")
 widget_control, last, get_value=endslice
 step=widget_info(Self.inbase, find_by_uname="wid_text_48")
 widget_control, step, get_value=stbi
 
 filled=widget_info(Self.inbase, find_by_uname="wid_button_50")
 widget_control, filled, get_value=fc

 if obj_valid(self.opanel) then obj_destroy, self.opanel

;
; Values read from the widgets are as arrays of strings, and not as integer. 
; make them integers, and not arrays
; 
 bid=fix(bid)
 eid=fix(eid)
 tbi=fix(tbi)
 endslice=fix(endslice)
 beginslice=fix(beginslice)
 stbi=fix(stbi)
 bid=bid[0]
 eid=eid[0]
 tbi=tbi[0]
 endslice=endslice[0]
 beginslice=beginslice[0]
 stbi=stbi[0]

;
; Depending on the direction of the cut, set max and min values and titles
; Along = 0: Cut perpendicular to the jet axis
; Along = 1: Cut along the jet axis
; Along = 2: Cut along the jet axis
;
 
  if dim eq '2' then begin
   along = 2
  endif

 if along eq 0 then begin
   Xmin = DimscaleY(0)
   Xmax = DimscaleY(Dimsizes(1)-1)
   ymin = DimscaleZ(0)
   ymax = DimscaleZ(Dimsizes(2)-1)
   dims1 = dimscaley
   dims2 = dimscalez
   xtitle= "x2"
   ytitle= "x3"
 endif

 if along eq 1 then begin
   Xmin = DimscaleX(0)
   Xmax = DimscaleX(Dimsizes(0)-1)
   ymin = DimscaleZ(0)
   ymax = DimscaleZ(Dimsizes(2)-1)
   dims1 = dimscalex
   dims2 = dimscalez
   xtitle= "x1"
   ytitle= "x3"
 endif

 if along eq 2 then begin
   Xmin = DimscaleX(0)
   Xmax = DimscaleX(Dimsizes(0)-1)
   ymin = DimscaleY(0)
   ymax = DimscaleY(Dimsizes(1)-1)
   dims1 = dimscalex
   dims2 = dimscaley
   xtitle= "x1"
   ytitle= "x2"

 endif

;
; set usepanel to 0 or 1, depending on whether panel is chosen (1) or not (0)
;
 usepanel = 0
 if uvalue eq 'PANEL' then begin
   opanel = obj_new('grpanel', columns=n_columns,rows=n_rows)
   usepanel=1
 endif

;
; Scanning in time
;

 if self.cs eq 0 then begin
  maxminvar_1, (*self.vars)[self.droplist], bid, eid, fix(beginslice), q, tbi=tbi, along=aa
 
  for nid = bid, eid, tbi do begin
    data=getdata((*self.vars)[self.droplist], nid)
    max=fmax
    min=fmin
;
; Get the right data
;
    if dim eq '3' then begin
         IF along eq 0 THEN Plotdata = REFORM(data[beginslice,*,*])
         IF along eq 1 THEN Plotdata = REFORM(data[*,beginslice,*])
         IF along eq 2 THEN Plotdata = REFORM(data[*,*,beginslice])
    endif 
    if dim eq '2' then begin
         plotdata = data[*,*]
    endif
    view=obj_new('idlgrview')
    view->setproperty,VIEWPLANE_RECT=[Xmin-(xmax-xmin)*0.20,Ymin-(ymax-ymin)*0.15,(Xmax-Xmin)*1.4,(Ymax-Ymin)*1.3], COLOR=[255,255,255]
;
; Filled plot chosen:
;
    if fc eq 1 then begin
       mContour = OBJ_NEW('grContour', Plotdata, Xmin=xmin, Xmax=xmax,Ymin=ymin,Ymax=ymax, $
       BTitle=strtrim(string((*self.vars)[self.droplist]))+': max='+strtrim(string(max),1)+' min='+strtrim(string(min),1), $
       xtitle=xtitle, $
       YTitle=YTitle, max=fmax, min=fmin, /color, /FILL, ct=self.palette_droplist, $
       title="Time = "+strtrim(string(nid*dt),1)+"  Slice = "+strtrim(string(beginSlice),1), $
       usepanel=usepanel, dims1=dims1, dims2=dims2)
    endif
;
; Contour plot chosen:
;
    if fc eq 0 then begin
       mContour = OBJ_NEW('grContour', Plotdata, Xmin=xmin,Xmax=xmax,Ymin=ymin,Ymax=ymax, $
       BTitle=strtrim(string((*self.vars)[self.droplist]))+': max='+strtrim(string(max),1)+' min='+strtrim(string(min),1), $ 
       xtitle=xtitle, $
       YTitle=YTitle, max=fmax, min=fmin, title="Time = "+strtrim(string(nid*dt), 1)+"  Slice="+strtrim(string(beginSlice),1), $
       usepanel=usepanel, /nocolorbar, dims1=dims1, dims2=dims2)
    endif

;
; Put the objects together
;
    omodel=obj_new('idlgrmodel')
    omodel->add, mcontour
    View->Add, omodel
    if uvalue eq 'PANEL' then begin
      opanel -> add, view
    endif else begin
      Self->resize,xstretch=1.1
      Self->draw,View
    endelse
;
; free up some memory
;
    plotdata=0
    data=0
 endfor

 endif

;
; Scan in space, pretty much the same code as above, but the loop is different
;
 if self.cs eq 1 then begin
   maxminvar_1, (*self.vars)[self.droplist], bid, bid, fix(beginslice), q, tbi=tbi, along=aa, endslice=fix(endslice), sbi=stbi
   data=getdata((*Self.Vars)[self.droplist], bid)
   for Slice=BeginSlice, EndSlice, STBI DO BEGIN
     max=fmax
     min=fmin
;
; Get the right data
;
     IF along eq 0 THEN Plotdata = REFORM(data[slice,*,*])
     IF along eq 1 THEN Plotdata = REFORM(data[*,slice,*])
     IF along eq 2 THEN Plotdata = REFORM(data[*,*,slice])
     if dim eq '2' then plotdata = (data[*,*])
     view=obj_new('idlgrview')

     view->setproperty,VIEWPLANE_RECT=[Xmin-(xmax-xmin)*0.20,Ymin-(ymax-ymin)*0.15,(Xmax-Xmin)*1.4,(Ymax-Ymin)*1.3], COLOR=[255,255,255]
;
; Filled plot
;
     if fc eq 1 then begin
       mContour = OBJ_NEW('grContour', Plotdata, Xmin=xmin, Xmax=xmax,Ymin=ymin,Ymax=ymax, $
       BTitle=strtrim(string((*Self.Vars)[self.droplist]))+': max='+strtrim(string(max),1)+' min='+strtrim(string(min),1), $ 
       xtitle=xtitle, $
       YTitle=YTitle, max=fmax, min=fmin, /color, /FILL, ct=self.palette_droplist, $ 
       title="Time = "+strtrim(string(bid*dt),1)+"  Slice = "+strtrim(string(Slice),1), $ 
       usepanel=usepanel, dims1=dims1, dims2=dims2);,min_value=35.0, max_value=65.0)
     endif
;
; Contour plot
;
     if fc eq 0 then begin
       mContour = OBJ_NEW('grContour', Plotdata, Xmin=xmin,Xmax=xmax,Ymin=ymin,Ymax=ymax, $
       BTitle=strtrim(string((*Self.Vars)[self.droplist]))+': max='+strtrim(string(max),1)+' min='+strtrim(string(min),1), $ 
       xtitle=xtitle, $
       YTitle=YTitle, max=fmax, min=fmin, ct=pal, title="Time = "+strtrim(string(bid*dt),1)+"  Slice="+strtrim(string(Slice),1) , $ 
       usepanel=usepanel, /nocolorbar, dims1=dims1, dims2=dims2)
     endif
;
; Put the objects together
;
     View->Add, mContour
     if uvalue eq 'PANEL' then begin
      opanel -> add, view
     endif else begin
      Self->draw,View
      if self.mov eq 1 then begin
        image=Self->read()
        movie->put,image,(nid-bid)*30
      endif
     endelse
     plotdata=0
   endfor
   data=0
 endif
;
; end of scan in time/space if's. The following is independent of how the
;  scanning is performed
;
 if uvalue eq 'PANEL' then  begin
   self->resize, xstretch=n_columns/float(n_rows)
   self->draw, opanel
 endif

 print, "Finished the Scan/Slices module"
END

PRO slices::up
 common setupvariables
; 
; A procedure to blank out some widgets and unblank others, depending on the
;  choices being made.
;
; If the dimension is 2, there is no slice to choose, thus they are cleared.
; Also, scanning in space does not make sense in 2 dimensions.
;
 case dim of
    '2'    :  begin
      cclear1=widget_info(self.inbase, find_by_uname="wid_button_34")
      widget_control, cclear1, sensitive=0
      cclear2=widget_info(self.inbase, find_by_uname="wid_button_36")
      widget_control, cclear2, sensitive=0
      cclear3=widget_info(self.inbase, find_by_uname="wid_text_46")
      widget_control, cclear3, sensitive=0
      cclear4=widget_info(self.inbase, find_by_uname="wid_text_47")
      widget_control, cclear4, sensitive=0
      cclear5=widget_info(self.inbase, find_by_uname="wid_text_48")
      widget_control, cclear5, sensitive=0
      unclear1=widget_info(self.inbase, find_by_uname="wid_text_31")
      widget_control, unclear1, sensitive=1
      unclear2=widget_info(self.inbase, find_by_uname="wid_text_32")
      widget_control, unclear2, sensitive=1
    end
;
; Make sure the widgets are not blanked out in 3-d.
;
    '3'    : begin
      cclear1=widget_info(self.inbase, find_by_uname="wid_button_34")
      widget_control, cclear1, sensitive=1
      cclear2=widget_info(self.inbase, find_by_uname="wid_button_36")
      widget_control, cclear2, sensitive=1
      cclear3=widget_info(self.inbase, find_by_uname="wid_text_46")
      widget_control, cclear3, sensitive=1
      cclear4=widget_info(self.inbase, find_by_uname="wid_text_47")
      widget_control, cclear4, sensitive=1
      cclear5=widget_info(self.inbase, find_by_uname="wid_text_48")
      widget_control, cclear5, sensitive=1
    end
    '2.5'    : begin
      cclear1=widget_info(self.inbase, find_by_uname="wid_button_34")
      widget_control, cclear1, sensitive=1
      cclear2=widget_info(self.inbase, find_by_uname="wid_button_36")
      widget_control, cclear2, sensitive=1
      cclear3=widget_info(self.inbase, find_by_uname="wid_text_46")
      widget_control, cclear3, sensitive=1
      cclear4=widget_info(self.inbase, find_by_uname="wid_text_47")
      widget_control, cclear4, sensitive=1
      cclear5=widget_info(self.inbase, find_by_uname="wid_text_48")
      widget_control, cclear5, sensitive=1
    end
 endcase

;
; If scanning in time (self.cs = 0), blank out some of the widgets related
; to scanning in space. Unblank those related to scanning in time.
;
   case self.cs of
     0     : begin
      clear1=widget_info(self.inbase, find_by_uname="wid_text_47")
      widget_control, clear1, sensitive=0
      clear2=widget_info(self.inbase, find_by_uname="wid_text_48")
      widget_control, clear2, sensitive=0
      unclear1=widget_info(self.inbase, find_by_uname="wid_text_31")
      widget_control, unclear1, sensitive=1
      unclear2=widget_info(self.inbase, find_by_uname="wid_text_32")
      widget_control, unclear2, sensitive=1
     end
;
; If scanning in space (self.cs = 1), blank out some widgets related to 
; scanning in time. Unblank those related to scanning in space.
;
     1     : begin
      if dim eq '3' then begin
        clear1=widget_info(self.inbase, find_by_uname="wid_text_31")
        widget_control, clear1, sensitive=0
        clear2=widget_info(self.inbase, find_by_uname="wid_text_32")
        widget_control, clear2, sensitive=0
        unclear1=widget_info(self.inbase, find_by_uname="wid_text_47")
        widget_control, unclear1, sensitive=1
        unclear2=widget_info(self.inbase, find_by_uname="wid_text_48")
        widget_control, unclear2, sensitive=1
      endif
     end
 endcase
END

FUNCTION slices::init

; 
; Initializing function for the Scan/Slice module.
;

  status = self->Tool::Init('Slices')

  common new_variables
  common setupvariables

;
; Make the variables pointer point the right place
;
  self.vars = ptr_new(1)
  *Self.Vars = [varlist]
  if n_elements(extravars) gt 0 then *Self.Vars = [varlist, extravars]
  if n_elements(variables) gt 0 then begin
    *Self.Vars = [*Self.Vars, varlabels]
  endif

;
; The different widgets:
;
  WID_BASE_EXTRA2 = Widget_base(self.inbase, UNAME='WID_BASE_EXTRA' ,FRAME=0$
      ,XPAD=2 ,YPAD=2 ,ROW=2)

  WID_LABEL_66 = Widget_Label(WID_BASE_extra2, UNAME='WID_LABEL_66', $ 
    /ALIGN_LEFT ,VALUE='Scan')
;
; The radio buttons
;
  wid_button_34=cw_bgroup(wid_base_extra2, ["Time", "Space"],row=1, $ 
    /exclusive, /no_release, uname="wid_button_34", uvalue="wid_button_34")
;
; the variable droplist
;
  droplist = FSC_Droplist(Self.inbase, Value=*Self.Vars, Index=0, $ 
    uname='droplist', UValue='Slices_droplist', title='Variable')

  WID_LABEL_41 = Widget_Label(self.inbase, UNAME='WID_LABEL_41'  $
      ,/ALIGN_LEFT ,VALUE='File ID(s)')
;
; the input fields for the file numbers:
;  
  wid_text_30=cw_field(self.inbase, uname='wid_text_30',title='First          ', value=0)
  wid_text_31=cw_field(self.inbase, uname='wid_text_31',title='Last           ', value=60)
  wid_text_32=cw_field(self.inbase, uname='wid_text_32',title='Increment      ', value=1)
;  
; the title for the Slice selection
;  
  WID_LABEL_45 = Widget_Label(self.inbase, UNAME='WID_LABEL_45'  $
       ,/ALIGN_LEFT ,VALUE='Slice(s)')

  WID_BASE_78 = Widget_Base(self.inbase, UNAME='WID_BASE_78'  $
       ,XPAD=3 ,YPAD=3 ,COLUMN=1)
;
; the x1, x2, x3 buttons
;
  wid_button_40=cw_bgroup(wid_base_78, ["x1", "x2", "x3"], row=1, /exclusive, /no_release, uname="wid_button_36", uvalue="wid_button_36")
;
; and then the input fields for the slice selection
;
  wid_text_46=cw_field(self.inbase, uname='wid_text_46',title='First          ', value=1)
  wid_text_47=cw_field(self.inbase, uname='wid_text_47',title='Last           ', value=100)
  wid_text_48=cw_field(self.inbase, uname='wid_text_48',title='Increment      ', value=1)

  WID_BASE_88 = Widget_Base(self.inbase, UNAME='WID_BASE_88'  $
       ,XPAD=3 ,YPAD=3 ,row=1)

  wid_button_50=cw_bgroup(self.inbase, ["Contour", "Fill"], row=1, /exclusive, /no_release, uname="wid_button_50", uvalue="wid_button_50" )
;
; The xpalette button
;
  xpalette_button = widget_button(self.inbase, value="Xpalette", uvalue="xpalette", uname="xpalette")
;
; base for the other buttons  
;
  WID_BASE_82 = Widget_Base(self.inbase, UNAME='WID_BASE_82' ,FRAME=1  $
      ,XPAD=3 ,YPAD=3 ,ROW=1 ,/GRID_LAYOUT)
;
; the buttons  
;
  WID_BUTTON_38 = Widget_Button(WID_BASE_82, Uvalue='Run'  $
      ,/ALIGN_CENTER ,VALUE='Run')
  
  WID_BUTTON_57 = Widget_Button(WID_BASE_82, Uvalue='PS'  $
      ,/ALIGN_CENTER ,VALUE='Save')

  WID_BUTTON_56 = Widget_Button(WID_BASE_82, Uvalue='PANEL'  $
      ,/ALIGN_CENTER ,VALUE='Panel')

  WID_BUTTON_55 = Widget_Button(WID_BASE_82, Uvalue='MPEG'  $
      ,/ALIGN_CENTER ,VALUE='MPEG')

  WID_BUTTON_39 = Widget_Button(WID_BASE_82, Uvalue='Help'  $
      ,/ALIGN_CENTER ,VALUE='Help')

  WID_BUTTON_40 = Widget_Button(WID_BASE_82, UNAME='WID_BUTTON_40'  $
      ,/ALIGN_CENTER ,VALUE='Close', uvalue='Close')

   widget_control, self.base, /update
   self->resize, xstretch=1.1
;
; Set the path for the Help file
;
   Self.HelpFile = "help/HELP_SLICES"
   return,1
END

PRO Slices::Set, helpfile = helpfile, mov = mov, cs = cs, droplist = droplist
;
; Procedure to set some variables
;
  if n_elements(cs) NE 0 then self.cs = cs
  if n_elements(droplist) NE 0 then Self.droplist = droplist
  if n_elements(helpfile) ne 0 then self.helpfile = helpfile
END

PRO slices__define
   void = {slices, droplist : 0,  palette_droplist : 0, $
           Vars: ptr_new(1), cs:2, mov:0, $ 
           INHERITS tool}
END
