;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
; 
; JETGET module for visualising vectors in 2 dimensional slices through a
; dataset. Uses the engine in ve__define.pro.
;
; Written by Jan Staff spring 2003.
;
; To be called from the JETGET main window.
;

PRO vectors_event, Event

    widget_control, event.id, get_uvalue = uvalue
    widget_control, event.top, get_uvalue=obj

    ud=widget_info(event.top, find_by_uname="wid_button_34")
    widget_control, ud, get_value=cs

;
; What selection has been made:
;
    case uvalue of
      'Run'                     : obj->Run, uvalue
      'PANEL'                   : begin
                  obj->reset_panel, n_columns, n_rows
                  obj->run, uvalue
      end
      'Vectors_droplist'        : obj->Set, droplist = event.(3)
      'wid_button_34'           : begin
                                  obj->Set, cs = event.(4)
                                  obj->up
                                  end
      'wid_button_36'           : begin
                                  end
      'MPEG'                    : begin
                                    obj->MPEG
                                  end
      else                      : obj->Tool_Event, event
    endcase


END

PRO vectors::run, uvalue
;
; Procedure that is started when the Run, Panel or MPEG button is pressed
;
 Common maxmin_1
 common time
 common setupvariables

 default, uvalue, " "
 if obj_valid(Self.oMPEG) then self.frame=0

;
; Read input values
;
 yd=widget_info(self.inbase, find_by_uname="wid_button_36")
 widget_control, yd, get_value=aa
 along = aa
 first=widget_info(Self.inbase, find_by_uname="wid_text_30") 
 widget_control, first, get_value=bid
 last=widget_info(Self.inbase, find_by_uname="wid_text_31") 
 widget_control, last, get_value=eid
 step=widget_info(Self.inbase, find_by_uname="wid_text_32") 
 widget_control, step, get_value=tbi
 first=widget_info(Self.inbase, find_by_uname="wid_text_46")
 widget_control, first, get_value=beginslice
 last=widget_info(Self.inbase, find_by_uname="wid_text_47")
 widget_control, last, get_value=endslice
 step=widget_info(Self.inbase, find_by_uname="wid_text_48")
 widget_control, step, get_value=stbi
 missing=widget_info(Self.inbase, find_by_uname="wid_text_49")
 widget_control, missing, get_value=mis1
 missing2=widget_info(Self.inbase, find_by_uname="wid_text_50")
 widget_control, missing2, get_value=mis2
 vlength=widget_info(Self.inbase, find_by_uname="wid_text_51")
 widget_control, vlength, get_value=len
 widget_control, widget_info(self.inbase, find_by_uname="wid_step"), $
       get_value=step
 
; 
; Values read from the widgets are as arrays, and not as integer. make them 
; integers, and not arrays
; 
 bid=fix(bid)
 eid=fix(eid)
 tbi=fix(tbi)
 endslice=fix(endslice)
 beginslice=fix(beginslice)
 stbi=fix(stbi)
 bid=bid[0]
 eid=eid[0]
 tbi=tbi[0]
 endslice=endslice[0]
 beginslice=beginslice[0]
 stbi=stbi[0]
 mis1=mis1[0]
 mis2=mis2[0]
 len=len[0]
 step = step[0]
 
;
; Depending on the direction of the cut, set max and min values and titles.
; Along = 0: Cut perpendicular to the jet axis
; Along = 1: Cut along the jet axis
; Along = 2: Cut along the jet axis
;

  if dim eq '2' then begin
   along = 2
  endif

 if along eq 0 then begin
   Xmin = DimscaleY(0)
   Xmax = DimscaleY(Dimsizes(1)-1)
   ymin = DimscaleZ(0)
   ymax = DimscaleZ(Dimsizes(2)-1)
   dims1 = dimscaley
   dims2 = dimscalez
   xtitle= "x2"
   ytitle= "x3"
 endif

 if along eq 1 then begin
   Xmin = DimscaleX(0)
   Xmax = DimscaleX(Dimsizes(0)-1)
   ymin = DimscaleZ(0)
   ymax = DimscaleZ(Dimsizes(2)-1)
   dims1 = dimscalex
   dims2 = dimscalez
   xtitle= "x1"
   ytitle= "x3"
 endif

 if along eq 2 then begin
   Xmin = DimscaleX(0)
   Xmax = DimscaleX(Dimsizes(0)-1)
   ymin = DimscaleY(0)
   ymax = DimscaleY(Dimsizes(1)-1)
   dims1 = dimscalex
   dims2 = dimscaley
   xtitle= "x1"
   ytitle= "x2"

 endif

;
; set usepanel to 0 or 1, depending on whether panel is chosen (1) or not (0)
;
 usepanel = 0
 if uvalue eq 'PANEL' then begin
   opanel = obj_new('grpanel', columns=n_columns,rows=n_rows)
   usepanel = 1
 endif

 if obj_valid(self.oldview) then obj_destroy, self.oldview
 if obj_valid(self.opanel) then obj_destroy, self.opanel

;
; Scanning in time
;

 if self.cs eq 0 then begin

  for nid = bid, eid, tbi do begin

;
; Get the right data
;
   if dim eq '3' then begin
     IF along eq 0 THEN begin
        data2=getdata((*Self.Vars)[self.droplist]+'2', nid)
        data3=getdata((*Self.Vars)[self.droplist]+'3', nid)
        Plotdata1 = REFORM(data2[beginslice,*,*])
        Plotdata2 = REFORM(data3[beginslice,*,*])
     ENDIF
     IF along eq 1 THEN BEGIN
        data1=getdata((*Self.Vars)[self.droplist]+'1', nid)
        data3=getdata((*Self.Vars)[self.droplist]+'3', nid)
        Plotdata1 = REFORM(data1[*,beginslice,*])
        Plotdata2 = REFORM(data3[*,beginslice,*])
     ENDIF
     IF along eq 2 THEN BEGIN
        data1=getdata((*Self.Vars)[self.droplist]+'1', nid)
        data2=getdata((*Self.Vars)[self.droplist]+'2', nid)
        Plotdata1 = REFORM(data1[*,*,beginslice])
        Plotdata2 = REFORM(data2[*,*,beginslice])
     ENDIF
   endif
   if dim eq '2' then begin
      data1=getdata((*Self.Vars)[self.droplist]+'1', nid)
      data2=getdata((*Self.Vars)[self.droplist]+'2', nid)
      Plotdata1 = REFORM(data1[*,*])
      Plotdata2 = REFORM(data2[*,*])
   ENDIF
     view=obj_new('idlgrview')
     view->setproperty,VIEWPLANE_RECT=[Xmin-(xmax-xmin)*0.20,Ymin-(ymax-ymin)*0.15,(Xmax-Xmin)*1.3,(Ymax-Ymin)*1.3], COLOR=[255,255,255]

;
; Generate the vector object
;
     vect = OBJ_NEW('ve', plotdata1, plotdata2, dims1, dims2, missing1=mis1, missing2=mis2, $
                          usepanel=usepanel, length=len, xmin=xmin, xmax=xmax, $
                          ymin=ymin, ymax=ymax, step=step, $ 
                          Btitle=strupcase((*Self.Vars)[self.droplist])+'-vectors', $
                          title="Time = "+strtrim(string(nid*dt),1)+"  Slice = "+strtrim(string(beginSlice),1), $
                          xtitle=xtitle, ytitle=ytitle)
;
; Put the objects together
; 
     if (obj_valid(vect)) then begin
       View->Add, vect
      if uvalue eq 'PANEL' then begin
       opanel -> add, view
      endif else begin
        Self->draw,View
      endelse
     endif
  endfor

 endif

;
; Scan in space, pretty much the same code as above, but different loop
; 
 if self.cs eq 1 then begin
   if dim eq '3' then begin
     for Slice=BeginSlice, EndSlice, STBI DO BEGIN
;
; Get the right data
;
      IF along eq 0 THEN begin
        data2=getdata((*Self.Vars)[self.droplist]+'2', bid)
        data3=getdata((*Self.Vars)[self.droplist]+'3', bid)
        Plotdata1 = REFORM(data2[slice,*,*])
        Plotdata2 = REFORM(data3[slice,*,*])
      ENDIF
      IF along eq 1 THEN BEGIN
        data1=getdata((*Self.Vars)[self.droplist]+'1', bid)
        data3=getdata((*Self.Vars)[self.droplist]+'3', bid)
        Plotdata1 = REFORM(data1[*,slice,*])
        Plotdata2 = REFORM(data3[*,slice,*])
      ENDIF
      IF along eq 2 THEN BEGIN
        data1=getdata((*Self.Vars)[self.droplist]+'1', bid)
        data2=getdata((*Self.Vars)[self.droplist]+'2', bid)
        Plotdata1 = REFORM(data1[*,*,slice])
        Plotdata2 = REFORM(data2[*,*,slice])
      ENDIF
;
; The view
; 
      view=obj_new('idlgrview')
      view->setproperty,VIEWPLANE_RECT=[Xmin-(xmax-xmin)*0.20,Ymin-(ymax-ymin)*0.15,(Xmax-Xmin)*1.3,(Ymax-Ymin)*1.3], COLOR=[255,255,255]
;
; The vector object
;
      vect = OBJ_NEW('ve', plotdata1, plotdata2, dims1, dims2, missing1=mis1, missing2=mis2, length=len, xmin=xmin, xmax=xmax, $
                          ymin=ymin, ymax=ymax, step=step, $ 
                          Btitle=strupcase((*Self.Vars)[self.droplist])+'-vectors', $
                          title="Time = "+strtrim(string(bid*dt),1)+"  Slice = "+strtrim(string(Slice),1), $ 
                          usepanel=usepanel, xtitle=xtitle, ytitle=ytitle)
;
; Put the objects together
;
      if (obj_valid(vect)) then begin
        View->Add, vect
        if uvalue eq 'PANEL' then begin
         opanel -> add, view
        endif else begin
         Self->draw,View
        endelse
      endif
     endfor
   endif

 endif

;
; End of time/space selection. If Panel was chosen, put view into the panel
; object.
;
 if uvalue eq 'PANEL' then begin
    self->resize, xstretch=n_columns/float(n_rows)
    self->draw, opanel
 endif

 print, "Finished"

END

PRO vectors::up

 common setupvariables
; 
; A procedure to blank out some widgets and unblank others, depending on the
;  choices being made.
;
; If the dimension is 2, there is no slice to choose, thus they are cleared.
; Also, scanning in space does not make sense in 2 dimensions.
;
 case dim of
   '2'    :  begin
      cclear1=widget_info(self.inbase, find_by_uname="wid_button_34")
      widget_control, cclear1, sensitive=0
      cclear2=widget_info(self.inbase, find_by_uname="wid_button_36")
      widget_control, cclear2, sensitive=0
      cclear3=widget_info(self.inbase, find_by_uname="wid_text_46")
      widget_control, cclear3, sensitive=0
      cclear4=widget_info(self.inbase, find_by_uname="wid_text_47")
      widget_control, cclear4, sensitive=0
      cclear5=widget_info(self.inbase, find_by_uname="wid_text_48")
      widget_control, cclear5, sensitive=0
      unclear1=widget_info(self.inbase, find_by_uname="wid_text_31")
      widget_control, unclear1, sensitive=1
      unclear2=widget_info(self.inbase, find_by_uname="wid_text_32")
      widget_control, unclear2, sensitive=1
    end
;
; Make sure the widgets are not blanked out in 3-d.
;
    '3'    : begin
      cclear1=widget_info(self.inbase, find_by_uname="wid_button_34")
      widget_control, cclear1, sensitive=1
      cclear2=widget_info(self.inbase, find_by_uname="wid_button_36")
      widget_control, cclear2, sensitive=1
      cclear3=widget_info(self.inbase, find_by_uname="wid_text_46")
      widget_control, cclear3, sensitive=1
      cclear4=widget_info(self.inbase, find_by_uname="wid_text_47")
      widget_control, cclear4, sensitive=1
      cclear5=widget_info(self.inbase, find_by_uname="wid_text_48")
      widget_control, cclear5, sensitive=1
    end
 endcase

;
; If scanning in time (self.cs = 0), blank out some of the widgets related
; to scanning in space. Unblank those related to scanning in time.
;
 case self.cs of
     0     : begin
      clear1=widget_info(self.inbase, find_by_uname="wid_text_47")
      widget_control, clear1, sensitive=0
      clear2=widget_info(self.inbase, find_by_uname="wid_text_48")
      widget_control, clear2, sensitive=0
      unclear1=widget_info(self.inbase, find_by_uname="wid_text_31")
      widget_control, unclear1, sensitive=1
      unclear2=widget_info(self.inbase, find_by_uname="wid_text_32")
      widget_control, unclear2, sensitive=1
     end
;
; If scanning in space (self.cs = 1), blank out some widgets related to
; scanning in time. Unblank those related to scanning in space.
;
     1     : begin
      if dim eq '3' then begin
        clear1=widget_info(self.inbase, find_by_uname="wid_text_31")
        widget_control, clear1, sensitive=0
        clear2=widget_info(self.inbase, find_by_uname="wid_text_32")
        widget_control, clear2, sensitive=0
        unclear1=widget_info(self.inbase, find_by_uname="wid_text_47")
        widget_control, unclear1, sensitive=1
        unclear2=widget_info(self.inbase, find_by_uname="wid_text_48")
        widget_control, unclear2, sensitive=1
      endif
     end
 endcase
END

FUNCTION vectors::init
;
; Initializing function for the Vectors module.
;
  common new_variables 
  common setupvariables

  status = self->Tool::Init('Vectors')
;
; Make the variables pointer point the right place
;
  Self.Vars = ptr_new(1)
  if n_elements(ownveclabels) gt 0 then begin
   if n_elements(veclabels2) ne 0 then begin
     *Self.Vars = [ownveclabels(where(ownveclabels ne '0')), $
       veclabels2(where(veclabels2 ne '0'))]
   endif else begin
     if (where(ownveclabels ne '0'))[0] ne -1 then begin
       *Self.Vars = [ownveclabels(where(ownveclabels ne '0'))]
     endif
   endelse
  endif else begin
    *Self.Vars = ['b','v']
  endelse
    if n_elements(veclabels2) ne 0 then begin 
     if (where(veclabels2 ne '0' and veclabels2 ne ''))[0] ne -1 then begin
       if n_elements(*self.vars) ne 0 then begin
         *Self.Vars = ['b','v',veclabels2(where(veclabels2 ne '0' and veclabels2 ne ''))]
       endif else begin
         *Self.Vars = [veclabels2(where(veclabels2 ne '0' and veclabels2 ne ''))]
       endelse
     endif
    endif

;
; The different widgets:
;
  WID_BASE_EXTRA2 = Widget_base(self.inbase, UNAME='WID_BASE_EXTRA' ,FRAME=0$
      ,XPAD=2 ,YPAD=2 ,ROW=2)
;
; the radiobuttons  
;
  WID_LABEL_66 = Widget_Label(WID_BASE_extra2, UNAME='WID_LABEL_66' ,/ALIGN_LEFT ,VALUE='Scan')

  wid_button_34=cw_bgroup(wid_base_extra2, ["Time", "Space"],row=1, /exclusive, /no_release, uname="wid_button_34", uvalue="wid_button_34")
;
; The variable droplist
;
  droplist = FSC_Droplist(Self.inbase, Value=*Self.Vars, Index=Self.droplist, $ 
    uname='droplist', UValue='Vectors_droplist', title='Variable')

  WID_LABEL_41 = Widget_Label(self.inbase, UNAME='WID_LABEL_41'  $
      ,/ALIGN_LEFT ,VALUE='File ID(s)')
;
; the input fields
;  
  wid_text_30=cw_field(self.inbase, uname='wid_text_30',title='First          ', value=0)
  wid_text_31=cw_field(self.inbase, uname='wid_text_31',title='Last           ', value=60)
  wid_text_32=cw_field(self.inbase, uname='wid_text_32',title='Increment      ', value=1)
;  
; the Slice(s) title
;  
  WID_LABEL_45 = Widget_Label(self.inbase, UNAME='WID_LABEL_45'  $
       ,/ALIGN_LEFT ,VALUE='Slice(s)')

  WID_BASE_78 = Widget_Base(self.inbase, UNAME='WID_BASE_78'  $
       ,XPAD=3 ,YPAD=3 ,COLUMN=1)
;
; radio buttons
;
  wid_button_40=cw_bgroup(wid_base_78, ["x1", "x2", "x3"], row=1, $
       /exclusive, /no_release, uname="wid_button_36", uvalue="wid_button_36")
;
; and then the input fields
;
  wid_text_46=cw_field(self.inbase, uname='wid_text_46',title='First          ', $ 
       value=1)
  wid_text_47=cw_field(self.inbase, uname='wid_text_47',title='Last           ', $ 
       value=100)
  wid_text_48=cw_field(self.inbase, uname='wid_text_48',title='Increment      ', $ 
       value=1)

  wid_label_length = widget_label(self.inbase, /align_left, $ 
       value="Vector:") 

  wid_text_51=cw_field(self.inbase, uname='wid_text_51',title='Length         ', $
       value=5)
  wid_step=cw_field(self.inbase, uname='wid_step', title='Step size      ', $ 
       value=7)

  wid_label_45 = widget_label(self.inbase, /align_left, value='Range:')
  wid_text_49=cw_field(self.inbase, uname='wid_text_49',title='Max            ', $
       value=0.3)
  wid_text_50=cw_field(self.inbase, uname='wid_text_50',title='Min            ', $ 
       value=0.0)


  WID_BASE_88 = Widget_Base(self.inbase, UNAME='WID_BASE_88'  $
       ,XPAD=3 ,YPAD=3 ,row=1)
;
; base for the buttons  
;
  WID_BASE_82 = Widget_Base(self.inbase, UNAME='WID_BASE_82' ,FRAME=1  $
      ,XPAD=3 ,YPAD=3 ,ROW=1 ,/GRID_LAYOUT)
;
; the buttons  
;
  WID_BUTTON_38 = Widget_Button(WID_BASE_82, Uvalue='Run'  $
      ,/ALIGN_CENTER ,VALUE='Run')
  
  WID_BUTTON_57 = Widget_Button(WID_BASE_82, Uvalue='PS'  $
      ,/ALIGN_CENTER ,VALUE='Save')
  
  WID_BUTTON_56 = Widget_Button(WID_BASE_82, Uvalue='PANEL'  $
      ,/ALIGN_CENTER ,VALUE='Panel')

  WID_BUTTON_55 = Widget_Button(WID_BASE_82, Uvalue='MPEG'  $
      ,/ALIGN_CENTER ,VALUE='MPEG')
  
  WID_BUTTON_39 = Widget_Button(WID_BASE_82, Uvalue='Help'  $
      ,/ALIGN_CENTER ,VALUE='Help')

  WID_BUTTON_40 = Widget_Button(WID_BASE_82, UNAME='WID_BUTTON_40'  $
      ,/ALIGN_CENTER ,VALUE='Close', uvalue='Close')

   widget_control, self.base, /update
   self->resize
;
; Set the path for the Help file
; 
   Self.HelpFile= "help/HELP_VECTORS"
   return,1
END


PRO Vectors::Set, mov = mov, cs = cs, droplist = droplist
;
; Procedure to set some variables
;
  if n_elements(cs) NE 0 then self.cs = cs
  if n_elements(droplist) NE 0 then Self.droplist = droplist

END

PRO vectors__define
   void = {vectors, droplist : 0,  dim_droplist : '3-D', $
           Vars: ptr_new(), cs:2, mov:0, $
           INHERITS tool}
END
